/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Components project.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

import Qt 4.7
import com.nokia.symbian 1.0
import com.nokia.symbian.themebridge 1.0

ImplicitSizeItem {
    id: progressBar

    // Common Public API
    property real minimumValue: 0.0
    property real maximumValue: 1.0
    property real value: 0.0
    property bool indeterminate: false

    implicitWidth: style.current.preferredWidth
    implicitHeight: style.current.preferredHeight

    Style {
        id: style
        styleClass: "ProgressBar"
    }

    Frame {
        id: background
        frameName: style.current.get("backgroundFrame")
        frameType: style.current.get("backgroundType")

        anchors.left: parent.left
        anchors.verticalCenter: parent.verticalCenter

        width: parent.width
        height: parent.height

        states: [
            State {
                name: "determinate"
                when: indeterminate == false
                PropertyChanges {
                    target: frame
                    visible: true
                }
                PropertyChanges {
                    target: indeterminateAnimation
                    visible: false
                }
                PropertyChanges {
                    target: propertyAnimation
                    running: false
                }
            },
            State {
                name: "indeterminate"
                when: indeterminate == true
                PropertyChanges {
                    target: frame
                    visible: false
                }
                PropertyChanges {
                    target: indeterminateAnimation
                    visible: true
                }
                PropertyChanges {
                    target: propertyAnimation
                    running: true
                }
            }
        ]

        PropertyAnimation { id: propertyAnimation; loops: Animation.Infinite; running: true; target: indeterminateAnimation; property: "animationOffset"; from: 0; to: 200; easing.type: Easing.Linear; duration: 5000 }

        Item {
            clip: true

            anchors.left: parent.left
            anchors.top: parent.top
            anchors.bottom: parent.bottom

            width: progressWidth()

            function progressWidth() {
                if (value <= minimumValue)
                    return 0;
                else if (value >= maximumValue)
                    return progressBar.width;
                else
                    return ((value - minimumValue) / (maximumValue - minimumValue)) * progressBar.width;
            }

            Frame {
                id: frame
                frameName: style.current.get("contentsFrame")
                frameType: style.current.get("contentsType")

                anchors.left: parent.left
                anchors.top: parent.top

                width: progressBar.width
                height: parent.height
            }
        }

        ProgressBarAnimation {
            id: indeterminateAnimation
            anchors.fill: parent

            animationIcon: style.current.get("indeterminateIcon")
            animationMask: style.current.get("indeterminateMask")
        }
    }
}

