/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Components project.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SDECLARATIVESCREEN_H
#define SDECLARATIVESCREEN_H

#include <QDeclarativeItem>

class SDeclarativeScreenPrivate;

class SDeclarativeScreen : public QObject
{
    Q_OBJECT

    Q_PROPERTY(Orientation orientation READ orientation WRITE setOrientation NOTIFY orientationChanged FINAL)
    Q_PROPERTY(QString orientationString READ orientationString NOTIFY orientationChanged FINAL)

    Q_PROPERTY(int width READ width NOTIFY sizeChanged)
    Q_PROPERTY(int height READ height NOTIFY sizeChanged)
    Q_PROPERTY(int rotation READ rotation NOTIFY orientationChanged FINAL)

    Q_PROPERTY(qreal ppi READ ppi)
    Q_PROPERTY(qreal ppmm READ ppmm)
    Q_PROPERTY(qreal unit READ unit NOTIFY displayChanged)
    Q_PROPERTY(QSizeF physicalSize READ physicalSize)

    Q_PROPERTY(bool minimized READ isMinimized WRITE setMinimized NOTIFY minimizedChanged FINAL)

    Q_ENUMS(Orientation)

public:
    explicit SDeclarativeScreen(QDeclarativeItem *parent = 0);
    virtual ~SDeclarativeScreen();

    enum Orientation {
        Automatic,
        Portrait,
        Landscape,
        PortraitInverted,
        LandscapeInverted
    };

    Orientation orientation() const;
    QString orientationString() const;
    void setOrientation(Orientation orientation);

    int rotation() const;

    int width() const;
    int height() const;

    bool isMinimized() const;
    void setMinimized(bool minimized);

    qreal ppi() const;
    qreal ppmm() const;
    qreal unit() const;
    QSizeF physicalSize() const;

Q_SIGNALS:
    void orientationChanged();
    void minimizedChanged();
    void sizeChanged();
    void displayChanged();

protected:
    QScopedPointer<SDeclarativeScreenPrivate> d_ptr;

private:
    Q_PRIVATE_SLOT(d_func(), void _q_updateOrientationAngle())
    Q_PRIVATE_SLOT(d_func(), void _q_updateScreenSize(const QSize&))

    Q_DISABLE_COPY(SDeclarativeScreen)
    Q_DECLARE_PRIVATE(SDeclarativeScreen)
};

QML_DECLARE_TYPE(SDeclarativeScreen)

#endif // SDECLARATIVESCREEN_H
