/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Components project.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SDECLARATIVEWINDOWDECORATION_H
#define SDECLARATIVEWINDOWDECORATION_H

#include <QDeclarativeItem>

class SDeclarativeWindowDecorationPrivate;

class SDeclarativeWindowDecoration : public QDeclarativeItem
{
    Q_OBJECT
    Q_PROPERTY(int orientation READ orientation WRITE setOrientation NOTIFY orientationChanged)
    Q_PROPERTY(bool backButtonVisible READ isBackButtonVisible WRITE setBackButtonVisible NOTIFY backButtonVisibleChanged)
    Q_PROPERTY(bool statusBarVisible READ isStatusBarVisible WRITE setStatusBarVisible NOTIFY statusBarVisibleChanged)
    Q_PROPERTY(bool titleBarVisible READ isTitleBarVisible WRITE setTitleBarVisible NOTIFY titleBarVisibleChanged)
    Q_PROPERTY(qreal topDecorationHeight READ topDecorationHeight WRITE setTopDecorationHeight NOTIFY topDecorationHeightChanged)
    Q_PROPERTY(qreal bottomDecorationHeight READ bottomDecorationHeight WRITE setBottomDecorationHeight NOTIFY bottomDecorationHeightChanged)
    Q_PROPERTY(QString title READ title WRITE setTitle NOTIFY titleChanged)
    Q_PROPERTY(QDeclarativeItem *toolBar READ toolBar WRITE setToolBar NOTIFY toolBarChanged)

public:
    explicit SDeclarativeWindowDecoration(QDeclarativeItem *parent = 0);
    ~SDeclarativeWindowDecoration();

    int orientation() const;
    void setOrientation(int orientation);

    bool isBackButtonVisible() const;
    void setBackButtonVisible(bool backButtonVisible);

    bool isStatusBarVisible() const;
    void setStatusBarVisible(bool statusBarVisible);

    bool isTitleBarVisible() const;
    void setTitleBarVisible(bool titleBarVisible);

    qreal topDecorationHeight() const;
    void setTopDecorationHeight(qreal topDecorationHeight);

    qreal bottomDecorationHeight() const;
    void setBottomDecorationHeight(qreal bottomDecorationHeight);

    QString title() const;
    void setTitle(const QString &title);

    QDeclarativeItem *toolBar() const;
    void setToolBar(QDeclarativeItem *toolBar);

    Q_INVOKABLE bool isLandscape() const;

#ifndef Q_OS_SYMBIAN
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
#endif

signals:
    void minimize();
    void quit();
    void backClicked();

    void orientationChanged(int orientation);
    void backButtonVisibleChanged(bool show);
    void statusBarVisibleChanged(bool visible);
    void titleBarVisibleChanged(bool visible);
    void topDecorationHeightChanged(qreal height);
    void bottomDecorationHeightChanged(qreal height);
    void titleChanged(const QString &title);
    void toolBarChanged(QDeclarativeItem *toolBar);

protected:
    QScopedPointer<SDeclarativeWindowDecorationPrivate> d_ptr;

private:
    Q_PRIVATE_SLOT(d_func(), void _q_doUpdateFullScreen())
    Q_PRIVATE_SLOT(d_func(), void _q_doUpdateCba())
    Q_PRIVATE_SLOT(d_func(), void _q_desktopWorkareaChanged())
    Q_PRIVATE_SLOT(d_func(), void _q_optionsSelected())
    Q_PRIVATE_SLOT(d_func(), void _q_exitSelected())
    Q_PRIVATE_SLOT(d_func(), void _q_backSelected())

    Q_DISABLE_COPY(SDeclarativeWindowDecoration)
    Q_DECLARE_PRIVATE(SDeclarativeWindowDecoration)
};

QML_DECLARE_TYPE(SDeclarativeWindowDecoration)

#endif // SDECLARATIVEWINDOWDECORATION_H
