/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Components project.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SSTYLEWRAPPER_H
#define SSTYLEWRAPPER_H

#include <QtCore/qobject.h>
#include <QtCore/qvariant.h>
#include <QtCore/qscopedpointer.h>
#include <QtGui/qcolor.h>
#include <QtGui/qfont.h>

class SStyleWrapperPrivate;

class SStyleWrapper : public QObject
{
    Q_OBJECT

    // To be set from QML
    Q_PROPERTY(QString mode READ mode WRITE setMode NOTIFY modeChanged)
    Q_PROPERTY(QString styleClass READ styleClass WRITE setStyleClass NOTIFY styleClassChanged)
    Q_PROPERTY(QString styleObjectName READ styleObjectName WRITE setStyleObjectName NOTIFY styleObjectNameChanged)

    // We want to export the currentStyle as a plain QObject. QML can use
    // its properties to get the style data.
    Q_PROPERTY(QObject *current READ currentStyleAsObject NOTIFY currentStyleChanged)

    // Some pre-processed data that we want to export to QML as well. This could be
    // calculated inside QML, its convenient to have it done in one place.
    Q_PROPERTY(qreal preferredWidth READ preferredWidth NOTIFY currentStyleChanged)
    Q_PROPERTY(qreal preferredHeight READ preferredHeight NOTIFY currentStyleChanged)

public:
    enum FontRole {
        Undefined = 0,
        Primary,
        Secondary,
        Title,
        PrimarySmall,
        Digital
    };


    SStyleWrapper(QObject *parent = 0);
    ~SStyleWrapper();

    QString mode() const;
    void setMode(const QString &mode);

    QString styleClass() const;
    void setStyleClass(const QString &styleClass);

    QString styleType() const;
    void setStyleType(const QString &styleType);

    QString styleObjectName() const;
    void setStyleObjectName(const QString &styleObjectName);

    QObject *currentStyleAsObject() const;

    qreal preferredWidth() const;
    qreal preferredHeight() const;

    Q_INVOKABLE void play(int effect);
    Q_INVOKABLE qreal textWidth(const QString &text, const QFont &font) const;
    Q_INVOKABLE qreal fontHeight(const QFont &font) const;
    Q_INVOKABLE QVariant get(const QString &propertyName);
    bool eventFilter(QObject *watched, QEvent *event);

Q_SIGNALS:
    void modeChanged();
    void styleClassChanged();
    void styleTypeChanged();
    void styleObjectNameChanged();
    void currentStyleChanged();

protected:
    QScopedPointer<SStyleWrapperPrivate> d_ptr;

private:
    Q_PRIVATE_SLOT(d_func(), void _q_desktopWorkareaChanged())
    Q_PRIVATE_SLOT(d_func(), void _q_displayChanged())

    Q_DISABLE_COPY(SStyleWrapper)
    Q_DECLARE_PRIVATE(SStyleWrapper)
};

#endif // SSTYLEWRAPPER_H
