/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Components project.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SSTYLEWRAPPER_P_H
#define SSTYLEWRAPPER_P_H

#include "sstylewrapper.h"

class SStyleWrapperPrivate
{
    Q_DECLARE_PUBLIC(SStyleWrapper)

public:
    SStyleWrapperPrivate(SStyleWrapper *qq);
    ~SStyleWrapperPrivate();

    QFont fetchFont(SStyleWrapper::FontRole role, qreal textPaneHeight = -1) const;
    QColor fetchThemeColor(const QString &colorName) const;
    qreal fetchLayoutParameter(const QString &layoutParameterName) const;
    void displayChanged();
    void play(int effect);
    void updateStyle();
    void invalidateStyle();
    void initScreenPtr() const;

    QVariant buttonProperty(const QString &propertyName) const;
    QVariant textEditProperty(const QString &propertyName) const;
    QVariant titleBarProperty(const QString &propertyName) const;
    QVariant listItemProperty(const QString &propertyName) const;
    QVariant basicListItemImageProperty(const QString &propertyName) const;
    QVariant pageContainerProperty(const QString &propertyName) const;
    QVariant checkBoxProperty(const QString &propertyName) const;
    QVariant dialogProperty(const QString &propertyName) const;
    QVariant scrollBarProperty(const QString &propertyName) const;
    QVariant progressBarProperty(const QString &propertyName) const;
    QVariant radioButtonProperty(const QString &propertyName) const;
    QVariant toolTipProperty(const QString &propertyName) const;

    void _q_desktopWorkareaChanged();
    void _q_displayChanged();

    SStyleWrapper *q_ptr;
    QString mode;
    QString styleClass;
    QString styleType;
    QString styleObjectName;

private:
    // Helpers
    QVariant buttonPropertyBackground() const;
    QVariant buttonPropertyTextColor() const;
    QVariant textEditPropertyFrame() const;
    QVariant listItemPropertyMainTextColor() const;
    QVariant listItemPropertySubTextColor() const;

private:
    mutable bool updateOnScreenAreaChange;
    mutable bool listeningDisplayChange;
};

#endif // SSTYLEWRAPPER_P_H
